# common.tcl --
#
#	Plugin main and remoted common parts (setup).
#
# CONTACT:	tclplugin-core@lists.sourceforge.net
#
# ORIGINAL AUTHORS:	Jacob Levy		Laurent Demailly
#
# Copyright (c) 1996-1997 Sun Microsystems, Inc.
# Copyright (c) 2000 by Scriptics Corporation.
# Copyright (c) 2002-2004 ActiveState Corporation.
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# RCS:  @(#) $Id: common.tcl,v 1.4 2004/05/18 17:07:31 hobbs Exp $

# we provide browser functionalities:

package provide setup 1.0

# We require logging (we will initialize it from here):

package require pluglog 1.1

# tkInit --
#
# Override tkInitScript.h tkInit for the plugin.
# If it exists and thus will find directly the Tk where it is expected
# without being potentially confused by the TK_LIBRARY env var...
#
proc tkInit {} {
    ::pluglog::log {} "direct tkInit! -> $::tk_library"
    uplevel \#0 [list source [file join $::tk_library tk.tcl]]
    #rename tkInit {}
}

# If no widget have been created, we withdraw the unneeded .
if {![llength [winfo children .]]} {
    wm withdraw .
}

# Set our nice name (to be seen by other apps (inspect, tkcon,...))
tk appname $::Name

# Setup and eventually start logging facility:

proc SetupLogging {} {
    global env

    if {[info exists env(TCL_PLUGIN_TS)]} {
	proc ::pluglog::TS {} $env(TCL_PLUGIN_TS)
    }

    lappend ::pluglog::attributes SLAVE {-background green -foreground black}

    # Start logging depending on environment vars.
    if {[info exists env(TCL_PLUGIN_LOGFILE)] \
	    && ($env(TCL_PLUGIN_LOGFILE) != 0) } {
	::pluglog::setup $env(TCL_PLUGIN_LOGFILE)
    } elseif {([info exists env(TCL_PLUGIN_LOGWINDOW)]) \
	    && ($env(TCL_PLUGIN_LOGWINDOW) != 0)} {
	::pluglog::setup window "Log: $::Name"
    }
}

# (Eventually) Set up a console if the user wants one:

proc SetupConsole {} {
    global env plugin

    # Create a console if the user asks for it:

    ::pluglog::log SetupConsole "Console setup"
    if {[info exists env(TCL_PLUGIN_CONSOLE)] \
	    && ($env(TCL_PLUGIN_CONSOLE) != 0)} {
	if {($env(TCL_PLUGIN_CONSOLE) == 1) || \
		($env(TCL_PLUGIN_CONSOLE) == "")} {
	    set consoleFile [file join $plugin(library) .. tkcon tkcon.tcl]
	    set cmd {
		namespace eval ::tkcon {}
		set ::tkcon::PRIV(protocol) {tkcon hide}
		set ::tkcon::OPT(exec) ""
		tkcon show
	    }
	} else {
	    set consoleFile $env(TCL_PLUGIN_CONSOLE)
	}

	# these may not exist
	if {![info exists ::argv]} { set ::argv {}; set ::argc 0 }

	if {[catch {uplevel \#0 [list source $consoleFile]} msg]} {
	    NotifyError SetupConsole\
		"SetupConsole loading \"$consoleFile\": $msg\n$::errorInfo"
	    return
	}
	if {[info exists $cmd] && [catch {uplevel \#0 $cmd} msg]} {
	    NotifyError SetupConsole\
		"SetupConsole \"$cmd\": $msg\n$::errorInfo"
	    return
	}
	::pluglog::log SetupConsole "Console file \"$consoleFile\" sourced ok"
    } else {
	::pluglog::log SetupConsole "No console created"
    }
}


# Notify user of a critical error:

proc NotifyError {name msg} {
    ::pluglog::log $name $msg ERROR

    tk_messageBox -icon error -title "Error: $::Name"\
	    -message "$name: $msg" -type ok
}

# Configuration setup

proc SetupConfig {} {
    global plugin

    # Initialiaze the configuration (install time / raw parameters):

    package require cfg 1.0

    # one thing the installed.cfg is supposed to do is
    # to set a value for plugin(executable). initialize here it just in case.

    set plugin(executable) {}

    # We will register in the "plugin" slot but we use the semi generated
    # installed.cfg file which will point to the customizable userConfig
    # file (which is supposed to be "plugin" too but in the config/ dir)
    set configFile [file join $plugin(library) installed.cfg]
    if {[catch {::cfg::init "plugin" $configFile} msg]} {
	NotifyError Init "$msg: probable plugin install problem ($configFile)"
    }
    # Now re-init the config with the userConfig
    if {[catch {::cfg::init $::cfg::userConfig} msg]} {
	::pluglog::log {} "$msg: probable plugin configuration problem" ERROR
    }
}
